
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/device.h>
#include "ssd_debug.h"

/* replace this with a proper division of the include files */
#include "sdiodrv.h"




MODULE_LICENSE("GPL");
MODULE_VERSION("0.99.0");
MODULE_DESCRIPTION("Legacy SDIO with Shared SDIO interface for DaVinci boards");
MODULE_AUTHOR("Yanir Lubetkin");
/*MODULE_ALIAS(""); */
/*MODULE_DEVICE_TABLE(""); */

MODULE_PARM(ssd_cmd_response_timeout, "i");
MODULE_PARM_DESC(ssd_cmd_response_timeout, "command response timeout for sdio in milliseconds");
#ifdef SSD_DEBUG
MODULE_PARM(ssd_debug_ip, "s");
MODULE_PARM_DESC(ssd_debug_ip, "ip address for udp dump");
MODULE_PARM(ssd_debug_port, "i");
MODULE_PARM_DESC(ssd_debug_port, "port for udp dump");
MODULE_PARM(g_ssd_debug_level, "i");
MODULE_PARM_DESC(g_ssd_debug_level, "debug level");
MODULE_PARM(use_sock, "i");
MODULE_PARM_DESC(use_sock, "enables socket usage");

static char *ssd_debug_ip = "10.0.3.200";
int ssd_debug_port = 5555;
int g_ssd_debug_level = SSD_REPORTLEVEL;
int use_sock = 0;
void* debugcallback;
#else
int g_ssd_debug_level = 4;
#endif
int ssd_cmd_response_timeout = 250;
/* driver model stuff*/

static void ssd_release(struct device *dev)
{
	PDEBUG("entering %s()\n" , __FUNCTION__ );
	/* Nothing to release? */
	PDEBUG("exiting %s\n",__FUNCTION__);
}

static u64 ssd_dmamask = 0xffffffff;

static struct resource ssd_resources[] = {
	{
	 .start = IO_ADDRESS(MMCSD_REGS_BASE_ADDR),
	 .end = IO_ADDRESS((MMCSD_REGS_BASE_ADDR) + 0x74),
	 .flags = IORESOURCE_MEM,
	 },
	{
	 .start = MMCINT_INTERRUPT,
	 .flags = IORESOURCE_IRQ,
	 },
};

static struct platform_device ssd_device = {
	.name = "ssd",
	.id = 1,
	.dev = {
		.release = ssd_release,
		.dma_mask = &ssd_dmamask,
		},
	.num_resources = ARRAY_SIZE(&ssd_resources),
	.resource = ssd_resources,
};


static struct device_driver ssd_driver = {
	.name = SDIO_DRIVER_NAME,
	.bus = &platform_bus_type,
};


/*
	module initialization and shutdown calls
*/

static void __exit ssd_shutdown(void)
{
/* Cleanup code here */
	PDEBUG("entering %s()\n" , __FUNCTION__ );
	ssd_close_socket();
	driver_unregister(&ssd_driver);
	platform_device_unregister(&ssd_device);
	PDEBUG("exiting %s\n",__FUNCTION__);
}


static int __init ssd_init(void)
{
	int ret;
	PDEBUG("entering %s()\n" , __FUNCTION__ );
	ret = ssd_init_socket(ssd_debug_ip,ssd_debug_port);
	if (ret != 0){
		PERR("failed openning debug socket\n");
		ssd_shutdown();
		return(-ENODEV);
	}
#ifdef SSD_DEBUG
	do{
		char buf[10] = {0xaa,0xbb,0xcc,0xdd,0xee,0xff,0x11,0x22,0x33,0x44};
		printk("sending test packet to %s:%d\n",ssd_debug_ip,ssd_debug_port);
		ssd_send_as_hexdump(buf,10,0);
		ssd_send_as_hexdump(buf,10,1);
	}while(0);
#endif	
/* registration with the system */
	ret = platform_device_register(&ssd_device);
	if (ret != 0){
		PERR("failed registering platform device\n");
		ssd_shutdown();
		return(-ENODEV);
	}
	ret = driver_register(&ssd_driver);
	if (ret != 0){
		PERR("failed registering driver\n");
		ssd_shutdown();
		return(-ENODEV);
	}
	PDEBUG("exiting %s\n",__FUNCTION__);
	return 0;
}


module_init(ssd_init);
module_exit(ssd_shutdown);
