#include <linux/types.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/netfilter.h>
#include <linux/in.h>
#if defined(NAT_STUN_SUPPORT) || defined(CONFIG_IP_NF_SIP)
#include <linux/ip.h>
#endif
#include <linux/udp.h>
#include <linux/netfilter_ipv4/ip_conntrack_protocol.h>

unsigned long ip_ct_udp_timeout = 30*HZ;
unsigned long ip_ct_udp_timeout_stream = 180*HZ;
/* 0611121: NAT Stun Support: Sumedh */
/* NOTE: the chanes with the tag NAT_STUN_SUPPORT in this file are responsible for:
 * 	a) Keep the STUN conntrack of the phone alive fot about 30mins
 *	   This ensures that the wan_port binding remains and the same is used for SIP sessions
*/
#ifdef NAT_STUN_SUPPORT
#define UDP_STREAM_INF_TIMEOUT (1800*HZ)
//#define STUN_SERVER_PORT 3478
extern int stun_server_port;
extern int stun_server_port2;
#endif

static int udp_pkt_to_tuple(const void *datah, size_t datalen,
			    struct ip_conntrack_tuple *tuple)
{
	const struct udphdr *hdr = datah;

	tuple->src.u.udp.port = hdr->source;
	tuple->dst.u.udp.port = hdr->dest;

	return 1;
}

static int udp_invert_tuple(struct ip_conntrack_tuple *tuple,
			    const struct ip_conntrack_tuple *orig)
{
	tuple->src.u.udp.port = orig->dst.u.udp.port;
	tuple->dst.u.udp.port = orig->src.u.udp.port;
	return 1;
}

/* Print out the per-protocol part of the tuple. */
static unsigned int udp_print_tuple(char *buffer,
				    const struct ip_conntrack_tuple *tuple)
{
	return sprintf(buffer, "sport=%hu dport=%hu ",
		       ntohs(tuple->src.u.udp.port),
		       ntohs(tuple->dst.u.udp.port));
}

/* Print out the private part of the conntrack. */
static unsigned int udp_print_conntrack(char *buffer,
					const struct ip_conntrack *conntrack)
{
	return 0;
}

/* Returns verdict for packet, and may modify conntracktype */
static int udp_packet(struct ip_conntrack *conntrack,
		      struct iphdr *iph, size_t len,
		      enum ip_conntrack_info conntrackinfo)
{
	/* If we've seen traffic both ways, this is some kind of UDP
	   stream.  Extend timeout. */
	if (test_bit(IPS_SEEN_REPLY_BIT, &conntrack->status)) {
#ifdef NAT_STUN_SUPPORT
		if( ntohs(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.udp.port) == stun_server_port ||
				ntohs(conntrack->tuplehash[IP_CT_DIR_REPLY].tuple.dst.u.udp.port) == stun_server_port ||
				ntohs(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.udp.port) == stun_server_port2 ||
				ntohs(conntrack->tuplehash[IP_CT_DIR_REPLY].tuple.dst.u.udp.port) == stun_server_port2 )
		{
			ip_ct_refresh(conntrack, UDP_STREAM_INF_TIMEOUT);
		}
		else
#endif
		ip_ct_refresh(conntrack, ip_ct_udp_timeout_stream);
		/* Also, more likely to be important, and not a probe */
		set_bit(IPS_ASSURED_BIT, &conntrack->status);
	} else
	{
#ifdef NAT_STUN_SUPPORT
		if( ntohs(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.udp.port) == stun_server_port ||
				ntohs(conntrack->tuplehash[IP_CT_DIR_REPLY].tuple.dst.u.udp.port) == stun_server_port || 
				ntohs(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.udp.port) == stun_server_port2 ||
				ntohs(conntrack->tuplehash[IP_CT_DIR_REPLY].tuple.dst.u.udp.port) == stun_server_port2 )
		{
			ip_ct_refresh(conntrack, UDP_STREAM_INF_TIMEOUT);
		}
		else
#endif
		ip_ct_refresh(conntrack, ip_ct_udp_timeout);
	}

	return NF_ACCEPT;
}

/* Called when a new connection for this protocol found. */
static int udp_new(struct ip_conntrack *conntrack,
			     struct iphdr *iph, size_t len)
{
	return 1;
}

/* Nirav
 * There exist no function currently for udp_exp_matches_pkt.
 * This is there for TCP as there exist seq. no in the TCP header.
 * For the time being we use the checksum to match the exp with the current skb.
 */
static int udp_exp_matches_pkt(struct ip_conntrack_expect *exp,
			       struct sk_buff **pskb)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	const struct udphdr *udph = (const struct udphdr *)((u_int32_t *)iph + 5);

	if (exp->seq > 0) {
		if (exp->seq == ntohl(udph->len)) {
			//printk("exp->seq matches udph->len ret [1]\n");
			return 1;
		}
		else {
			//printk("exp->seq DOESNT match udph->len ret [0]\n");
			return 0;
		}
	}
	return 1;
}

struct ip_conntrack_protocol ip_conntrack_protocol_udp
= { { NULL, NULL }, IPPROTO_UDP, "udp",
    udp_pkt_to_tuple, udp_invert_tuple, udp_print_tuple, udp_print_conntrack,
    udp_packet, udp_new, NULL, udp_exp_matches_pkt, NULL };
    //udp_packet, udp_new, NULL, NULL, NULL };
